/*
 * Copyright 2025 ES4X
 *
 * ES4X licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

import { Handler, AsyncResult } from '@vertx/core';

import { BaseBridgeEvent } from '@vertx/bridge-common';
import { NetSocket } from '@vertx/core';

/**
 * Represents an event that occurs on the event bus bridge.
 * <p>
 * Please consult the documentation for a full explanation.
 */
export abstract class BridgeEvent extends BaseBridgeEvent {
  complete(result: boolean) : void;

  tryComplete(arg0: boolean) : boolean;

  future() : PromiseLike<boolean>;

  /**
   * Get the raw JSON message for the event. This will be null for SOCKET_CREATED or SOCKET_CLOSED events as there is
   * no message involved.
   */
  setRawMessage(message: { [key: string]: any }) : BridgeEvent;

  /**
   * Get the SockJSSocket instance corresponding to the event
   */
  socket() : NetSocket;
}

import { Vertx } from '@vertx/core';
import { NetServerOptions } from '@vertx/core/options';
import { BridgeOptions } from '@vertx/bridge-common/options';

/**
 * TCP EventBus bridge for Vert.x
 */
export abstract class TcpEventBusBridge {
  static create(vertx: Vertx) : TcpEventBusBridge;

  static create(vertx: Vertx, options: BridgeOptions) : TcpEventBusBridge;

  static create(vertx: Vertx, options: BridgeOptions, netServerOptions: NetServerOptions) : TcpEventBusBridge;

  static create(vertx: Vertx, options: BridgeOptions, netServerOptions: NetServerOptions, eventHandler: ((res: BridgeEvent) => void) | Handler<BridgeEvent>) : TcpEventBusBridge;

  /**
   * Listen on default port 7000 with a handler to report the state of the socket listen operation.
   */
  listen() : PromiseLike<TcpEventBusBridge>;

  /**
   * Listen on default port 7000 with a handler to report the state of the socket listen operation.
   */
  listen(handler: ((res: AsyncResult<TcpEventBusBridge>) => void) | Handler<AsyncResult<TcpEventBusBridge>>) : TcpEventBusBridge;

  /**
   * Listen on specific port and bind to specific address
   */
  listen(port: number, address: string) : PromiseLike<TcpEventBusBridge>;

  /**
   * Listen on specific port and bind to specific address
   */
  listen(port: number, address: string, handler: ((res: AsyncResult<TcpEventBusBridge>) => void) | Handler<AsyncResult<TcpEventBusBridge>>) : TcpEventBusBridge;

  /**
   * Listen on specific port
   */
  listen(port: number) : PromiseLike<TcpEventBusBridge>;

  /**
   * Listen on specific port
   */
  listen(port: number, handler: ((res: AsyncResult<TcpEventBusBridge>) => void) | Handler<AsyncResult<TcpEventBusBridge>>) : TcpEventBusBridge;

  /**
   * Close the current socket.
   */
  close() : PromiseLike<void>;

  /**
   * Close the current socket.
   */
  close(handler: ((res: AsyncResult<void>) => void) | Handler<AsyncResult<void>>) : void;
}
